@@ -1,17 +1,20 @@
-"""
-A collection of simplified utilities
-"""
+"""A collection of simplified utilities."""
+
 import pkg_resources as pkg  # part of setuptools
-import json
+#import json
 #import requests
 #from urllib.request import urlopen
 
 package = "smpl"
 
-version = pkg.require(package)[0].version
+try:
+    version = pkg.require(package)[0].version
+except pkg.DistributionNotFound:
+    version = "dirty"
+
 __version__ = version
 
-#def is_internet_available():
+# def is_internet_available():
 #    try:
 #        urlopen('http://216.58.192.142', timeout=1)
 #        return True
