@@ -1,37 +1,16 @@
-import matplotlib.patches as mpatches
 import numpy as np
-import statistics as stat
-import scipy as sci
-import scipy.integrate as integrate
-import scipy.fftpack
 import matplotlib.pyplot as plt
 import matplotlib
-import matplotlib.axes as axes
-from matplotlib import colors as mcolors
-import math
-from scipy import optimize
-import uncertainties as unc
+import uncertainties
 import uncertainties.unumpy as unp
-import uncertainties.umath as umath
-import glob
-import os
-import matplotlib.pyplot as plt
-from scipy.odr import *
 import sympy
-# from tqdm import tqdm
 import matplotlib.pylab as pylab
-import pathlib
-import types
-import inspect
 # local imports
-from smpl import functions
 from smpl import io
 from smpl import util
 from smpl import wrap
-from smpl.doc import append_doc, append_str
 from smpl import doc
 from smpl import fit as ffit
-import inspect
 
 
 def set_plot_style():
@@ -62,6 +41,7 @@ default = {
     'units': [None, "Units of the fit parameters as strings. Displayed in the Legend", ],
     'save': [None, " File to save the plot", ],
     'lpos': [0, "Legend position", ],
+    'tight': [True, "tight_layout", ],
     #          'frange'        :[None      ,"Limit the fit to given range. First integer is the lowest and second the highest index.",],
     'prange': [None, "Limit the plot of the fit to given range", ],
     'sigmas': [0, "Color the array of given ``sigma`` times uncertainty. Only works if the fit function is coded with ``unp``", ],
@@ -82,7 +62,7 @@ default = {
     # ,          'selector'      :[ None     ,"Function that takes ``x`` and ``y`` as parameters and returns an array mask in order to limit the data points for fitting. Alternatively a mask for selecting elements from datax and datay.",],
     # ,          'fixed_params'  :[ True     ,"Enable fixing parameters by choosing the same-named variables from ``kwargs``.",],
     # ,          'sortbyx'       :[ True     , "Enable sorting the x and y data so that x is sorted.",],
-    'interpolate': [True, "Enable interpolation of whole data if fit range is limited by ``frange`` or ``selector``.", ],
+    'interpolate': [True, "Enable interpolation of whole data if fit range is limited by ``frange`` or ``fselector``.", ],
     'interpolate_min': [None, "Lower interpolation bound", ],
     'interpolate_max': [None, "Higher interpolation bound", ],
     'interpolate_hatch': [r"||", "Interpolation shape/hatch for filled area in case of ``sigmas``>0. See https://matplotlib.org/stable/gallery/shapes_and_collections/hatch_style_reference.html", ],
@@ -93,17 +73,18 @@ default = {
     'grid': [True,  "Enable grid for the plot", ],
     'hist': [False, "Enable histogram plot", ],
     'stairs': [False, "Enable stair plot", ],
-
+    'capsize': [5, "size of cap on error bar plot"],
+    'axes': [None, "set current axis"]
 }
 
 
-# @doc.insert_str("\tDefault kwargs\n\n\t")
 @doc.append_doc(ffit.fit_kwargs)
-@doc.append_str(doc.table(default))
+@doc.append_str("\t")
+@doc.append_str(doc.table(default, init=False))
 @doc.append_str(doc.table({"plot_kwargs        ": ["default", "description"]}, bottom=False))
 def plot_kwargs(kwargs):
-    """
-    Set default plot_kwargs if not set.
+    """Set default plot_kwargs if not set.
+
     """
     kwargs = ffit.fit_kwargs(kwargs)
     for k, v in default.items():
@@ -118,14 +99,14 @@ def auto(datax, datay, funcs=None, **kwargs):
     Automatically loop over functions and fit the best one.
 
     Parameters
-    ==========
+    ----------
     funcs : function array
         functions to consider as fit. Default all ``smpl.functions``.
     **kwargs : optional
         see :func:`plot_kwargs`.
 
     Returns
-    =======
+    -------
     The best fit function and it's parameters. Also a lambda function where the parameters are already applied.
 
 
@@ -139,10 +120,11 @@ def auto(datax, datay, funcs=None, **kwargs):
 
 # @append_doc(default_kwargs)
 def fit(datax, datay, function, **kwargs):
-    """Fit and plot function to datax and datay.
+    """
+    Fit and plot function to datax and datay.
 
     Parameters
-    ==========
+    ----------
     datax : array_like
         X data either as ``unp.uarray`` or ``np.array`` or ``list``
     datay : array_like
@@ -154,12 +136,12 @@ def fit(datax, datay, function, **kwargs):
     Fit parameters can be fixed via ``kwargs`` eg. ``a=5``.
 
     Returns
-    =======
+    -------
     array_like
         Optimized fit parameters of ``function`` to ``datax`` and ``datay``
 
     Examples
-    ========
+    --------
 
     .. plot::
         :include-source:
@@ -192,10 +174,11 @@ def fit(datax, datay, function, **kwargs):
 
 
 def data(datax, datay, function=None, **kwargs):
-    """Plot datay against datax via :func:`fit`
+    """
+    Plot datay against datax via :func:`fit`
 
     Parameters
-    ==========
+    ----------
     datax : array_like
         X data either as ``unp.uarray`` or ``np.array`` or ``list``
     datay : array_like
@@ -205,7 +188,7 @@ def data(datax, datay, function=None, **kwargs):
     **kwargs : optional
         see :func:`plot_kwargs`.
     Returns
-    =======
+    -------
     array_like
         Optimized fit parameters of ``function`` to ``datax`` and ``datay``
     """
@@ -217,17 +200,34 @@ def data(datax, datay, function=None, **kwargs):
     return fit(datax, datay, function, **kwargs)
 
 
-def _plot(x, y, **kwargs):
-    args = [x, y]
+def _function(func, xfit, **kwargs):
     kargs = {}
     if util.has('fmt', kwargs):
-        args += [kwargs["fmt"]]
+        kargs["fmt"] = kwargs["fmt"]
     if util.has('label', kwargs) and kwargs['label'] != "":
         kargs['label'] = kwargs['label']
     if util.has('color', kwargs) and kwargs['color'] != "":
         kargs['color'] = kwargs['color']
-    plt.plot(*args, **kargs)
-# @append_doc(default_kwargs)
+    if util.has('sigmas', kwargs) and kwargs['sigmas'] != "":
+        kargs['sigmas'] = kwargs['sigmas']
+    __function(func, xfit, **kargs)
+
+
+def __function(gfunc, xlinspace, fmt="-", label=None, color=None, hatch=None, sigmas=0.):
+    func = gfunc
+    x = xlinspace
+    l = label
+    if isinstance(func(x[0]), uncertainties.UFloat):
+        if sigmas > 0:
+            ll, = plt.plot(x, unv(func(x)), fmt, color=color)
+            y = func(x)
+            plt.fill_between(x, unv(y)-sigmas*usd(y), unv(
+                y)+sigmas*usd(y), alpha=0.4, label=l, color=ll.get_color(), hatch=hatch)
+        else:
+            ll, = plt.plot(x, unv(func(x)), fmt, label=l, color=color)
+    else:
+        ll, = plt.plot(x, func(x), fmt, label=l, color=color)
+    return ll
 
 
 def function(func, *args, **kwargs):
@@ -235,7 +235,7 @@ def function(func, *args, **kwargs):
     Plot function ``func`` between ``xmin`` and ``xmax``
 
     Parameters
-    ==========
+    ----------
     func : function
         Function to be plotted between ``xmin`` and ``xmax``, only taking `array_like` ``x`` as parameter
     *args : optional
@@ -246,19 +246,21 @@ def function(func, *args, **kwargs):
     if not util.has("xmin", kwargs) or not util.has("xmin", kwargs):
         raise Exception("xmin or xmax missing.")
 
-    if not util.has('lpos', kwargs) and not util.has('label', kwargs):
-        kwargs['lpos'] = -1
+    # if not util.has('lpos', kwargs) and not util.has('label', kwargs):
+    #    kwargs['lpos'] = -1
     if not util.has('fmt', kwargs):
         kwargs['fmt'] = "-"
 
     kwargs = plot_kwargs(kwargs)
-    xfit = np.linspace(kwargs['xmin'], kwargs['xmax'], kwargs['steps'])
-    fig = init_plot(**kwargs)
+    xlin = np.linspace(kwargs['xmin'], kwargs['xmax'], kwargs['steps'])
+    init_plot(**kwargs)
 
     if not util.has("label", kwargs) or kwargs['label'] is None:
         kwargs['label'] = get_fnc_legend(func, args, **kwargs)
-        kwargs['lpos'] = 0
-    _plot(xfit, func(xfit, *args), **kwargs)
+        # kwargs['lpos'] = 0
+    #_plot(xfit, func(xfit, *args), **kwargs)
+    _function(wrap.get_lambda_argd(
+        func, kwargs['xvar'], *args), xlin, **kwargs)
     if kwargs['ss']:
         save_plot(**kwargs)
 
@@ -266,7 +268,7 @@ def function(func, *args, **kwargs):
 # xaxis="",yaxis="",fit_color=None,save = None,residue_err=True,show=False):
 def plt_residue(datax, datay, gfunction, fit, fig, **kwargs):
     function = wrap.get_lambda(gfunction, kwargs['xvar'])
-    frame2 = fig.add_axes((.1, .1, .8, .2))
+    fig.add_axes((.1, .1, .8, .2))
     kwargs['yaxis'] = "$\\Delta$" + kwargs['yaxis']
     kwargs['data_color'] = kwargs['fit_color']
 
@@ -288,10 +290,7 @@ def _fit(datax, datay, function, **kwargs):
     """
     return ffit.fit(datax, datay, function, **kwargs)
 
-# @append_doc(plot_kwargs)
 
-
-# xaxis="",yaxis="",label=None,fmt=None,data_color=None):
 def plt_data(datax, datay, **kwargs):
     """
         Plot datay vs datax
@@ -317,47 +316,32 @@ def plt_data(datax, datay, **kwargs):
                      color=kwargs['data_color'])
     else:
         if kwargs['fmt'] is None:
-            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=" ", capsize=5,
+            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=" ", capsize=kwargs["capsize"],
                          label=kwargs['label'], color=kwargs['data_color'])
         elif kwargs['fmt'] == "step":
             ll, = plt.step(x, y, where='mid',
                            color=kwargs['data_color'])
             if xerr is not None:
-                for ix in range(len(x)):
+                for ix, xv in enumerate(x):
                     dx = (xerr[ix])
-                    tx = [x[ix]-dx, x[ix]+dx]
+                    tx = [xv-dx, xv+dx]
                     plt.fill_between(tx, y[ix]-yerr[ix], y[ix]+yerr[ix],
                                      label=kwargs['label']if ix == 1 else None, alpha=0.2, step='pre', color=ll.get_color())
             else:
                 plt.fill_between(x, y-yerr, y+yerr,
                                  label=kwargs['label'], alpha=0.2, step='mid', color=ll.get_color())
         elif kwargs['fmt'] == "hist":
-            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=" ", capsize=5,
+            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=" ", capsize=kwargs["capsize"],
                          color="black")
             plt.fill_between(x, y, step="mid", label=kwargs['label'])
         else:
-            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=kwargs['fmt'], capsize=5,
+            plt.errorbar(x, y, yerr=yerr, xerr=xerr, fmt=kwargs['fmt'], capsize=kwargs["capsize"],
                          label=kwargs['label'], color=kwargs['data_color'])
 
 
 def get_fnc_legend(function, fit, **kwargs):
-    if isinstance(function, str):
-        l = "$" + sympy.latex(wrap.str_get_expr(function)) + "$"
-    else:
-        l = function.__name__
-    # l = ""
-    if l == "<lambda>":
-        # l = "$\\lambda$(" +  ','.join(function.__code__.co_varnames) + ") = " #sympy.latex(eval(function.__code__.co_code))
-        try:
-            cc, li = inspect.findsource(function)
-            f = ''.join(cc[li:]).split('lambda')[-1].split(':')[1].split(',')[
-                0].replace("\n", "")  # .replace("#", "").replace("'''","")
-            l = "$" + sympy.latex(wrap.str_get_expr(f)) + "$"
-        except OSError:
-            l = "$\\lambda$(" + ','.join(function.__code__.co_varnames) + ")"
+    l = wrap.get_latex(function)
 
-    if function.__doc__ is not None and not isinstance(function, str):
-        l = function.__doc__.split('\n')[0]
     vnames = wrap.get_varnames(function, kwargs['xvar'])
     for i in range(1, len(vnames)):
         l = l + ("\n" if not kwargs["fitinline"] or i == 1 else " ")
@@ -375,58 +359,45 @@ def get_fnc_legend(function, fit, **kwargs):
             l = l + " " + kwargs['units'][i-1]
     return l
 
-# @append_doc(default_kwargs)
 
-
-# p0=None,units=None,frange=None,prange=None,sigmas=1,residue=False, fig = None,fit_color=None):
 def plt_fit(datax, datay, gfunction, **kwargs):
     """
-       Plot Fit 
+    Plot Fit
     """
-    function = wrap.get_lambda(gfunction, kwargs['xvar'])
-    x, y, xerr, yerr = data_split(datax, datay, **kwargs)
+    func = wrap.get_lambda(gfunction, kwargs['xvar'])
     fit = _fit(datax, datay, gfunction, **kwargs)
+    def fitted(x): return func(x, *fit)
+    l = get_fnc_legend(gfunction, fit, **kwargs)
     if kwargs['prange'] is None:
+        x, _, _, _ = ffit.fit_split(datax, datay, **kwargs)
         xfit = np.linspace(np.min(unv(x)), np.max(unv(x)), 1000)
     else:
         xfit = np.linspace(kwargs['prange'][0], kwargs['prange'][1], 1000)
-    l = get_fnc_legend(gfunction, fit, **kwargs)
-    ll = None
-    if kwargs['sigmas'] > 0:
-        ll, = plt.plot(xfit, function(xfit, *unv(fit)),
-                       "-", color=kwargs['fit_color'])
-        yfit = function(xfit, *fit)
-        plt.fill_between(xfit, unv(yfit)-kwargs['sigmas']*usd(yfit), unv(
-            yfit)+kwargs['sigmas']*usd(yfit), alpha=0.4, label=l, color=ll.get_color())
-    else:
-        ll, = plt.plot(xfit, function(xfit, *unv(fit)), "-",
-                       label=l, color=kwargs['fit_color'])
-    if (kwargs['frange'] is not None or kwargs['selector'] is not None) and util.true('interpolate', kwargs) or util.has("interpolate_max", kwargs) or util.has("interpolate_min", kwargs):
+    ll = __function(fitted, xfit, "-", label=l,
+                    color=kwargs['fit_color'], sigmas=kwargs['sigmas'])
+
+    if (kwargs['frange'] is not None or kwargs['fselector'] is not None) and util.true('interpolate', kwargs) or util.has("interpolate_max", kwargs) or util.has("interpolate_min", kwargs):
         xxfit = np.linspace(util.get("interpolate_min", kwargs, np.min(
             unv(datax))), util.get("interpolate_max", kwargs, np.max(unv(datax))))
-        plt.plot(xxfit, unv(function(xxfit, *fit)), "--", color=ll.get_color())
-        if kwargs['sigmas'] > 0:
-            xxxfit = np.linspace(np.min(xxfit), np.min(xfit))
-            yfit = function(xxxfit, *fit)
-            plt.fill_between(xxxfit, unv(yfit)-kwargs['sigmas']*usd(yfit), unv(yfit)+kwargs['sigmas']*usd(
-                yfit), alpha=0.4, color=ll.get_color(), hatch=util.get("interpolate_hatch", kwargs, r"||"))
-            xxxfit = np.linspace(np.max(xfit), np.max(xxfit))
-            yfit = function(xxxfit, *fit)
-            plt.fill_between(xxxfit, unv(yfit)-kwargs['sigmas']*usd(yfit), unv(yfit)+kwargs['sigmas']*usd(
-                yfit), alpha=0.4, color=ll.get_color(), hatch=util.get("interpolate_hatch", kwargs, r"||"))
+        __function(fitted, np.linspace(np.min(xxfit), np.min(xfit)), "--",
+                   color=ll.get_color(), hatch=util.get("interpolate_hatch", kwargs, r"||"), sigmas=kwargs['sigmas'])
+        __function(fitted, np.linspace(np.max(xfit), np.max(xxfit)), "--",
+                   color=ll.get_color(), hatch=util.get("interpolate_hatch", kwargs, r"||"), sigmas=kwargs['sigmas'])
     return fit, ll.get_color()
 
 
-def init_plot(**kwargs):  # size=None,residue=False): #init
-    # fig = plt.figure(figsize=fig_size)
+def init_plot(**kwargs):
     fig = None
+    if util.has("axes", kwargs) and kwargs["axes"] is not None:
+        plt.sca(kwargs["axes"])
+        return kwargs["axes"].get_figure()
     if kwargs['init'] or util.true("residue", kwargs):
         if kwargs['size'] is None:
             fig = plt.figure()
         else:
             fig = plt.figure(figsize=kwargs['size'])
         if kwargs['residue']:
-            frame1 = fig.add_axes((.1, .3, .8, .6))
+            fig.add_axes((.1, .3, .8, .6))
     if util.has("xlabel", kwargs) and kwargs['xlabel'] != "":
         plt.xlabel(kwargs['xlabel'])
     if util.has("ylabel", kwargs) and kwargs['ylabel'] != "":
@@ -438,7 +409,7 @@ def init_plot(**kwargs):  # size=None,residue=False): #init
     return fig
 
 
-def save_plot(**kwargs):  # save=None,lpos=0,logy=False,logx=False,show=True): #save
+def save_plot(**kwargs):
     """
         save plot
     """
@@ -446,7 +417,8 @@ def save_plot(**kwargs):  # save=None,lpos=0,logy=False,logx=False,show=True): #
         plt.gca().set_yscale('log')
     if 'logx' in kwargs and kwargs['logx']:
         plt.gca().set_xscale('log')
-    plt.tight_layout()
+    if 'tight' in kwargs and kwargs['tight']:
+        plt.tight_layout()
     if 'lpos' in kwargs and kwargs['lpos'] >= 0:
         if(util.has('bbox_to_anchor', kwargs)):
             if(util.has('ncol', kwargs)):
